# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import WebDriverWait as Wait

from fxapom.fxapom import TIMEOUT
from fxapom.pages.base import Base


class SignIn(Base):

    _fox_logo_locator = (By.ID, 'fox-logo')
    _email_input_locator = (By.CSS_SELECTOR, '.input-row .email')
    _next_button_locator = (By.ID, 'email-button')
    _password_input_locator = (By.ID, 'password')
    _sign_in_locator = (By.ID, 'submit-btn')

    def __init__(self, driver, timeout=TIMEOUT):
        super(SignIn, self).__init__(driver, timeout)
        self._sign_in_window_handle = None
        self.popup = False
        self.check_for_popup(self.driver.window_handles)

    @property
    def login_password(self):
        """Get the value of the login password field."""
        return self.driver.find_element(*self._password_input_locator).get_attribute('value')

    @login_password.setter
    def login_password(self, value):
        """Set the value of the login password field."""
        password = self.driver.find_element(*self._password_input_locator)
        password.clear()
        password.send_keys(value)

    def click_next(self):
        self.driver.find_element(*self._next_button_locator).click()

    @property
    def email(self):
        """Get the value of the email field."""
        return self.driver.find_element(*self._email_input_locator).get_attribute('value')

    @email.setter
    def email(self, value):
        """Set the value of the email field."""
        email = Wait(self.driver, self.timeout).until(
            EC.visibility_of_element_located(self._email_input_locator))
        email.clear()
        email.send_keys(value)

    def check_for_popup(self, handles):
        if len(handles) > 1:
            self.popup = True
            for handle in handles:
                self.driver.switch_to.window(handle)
                if self.is_element_present(*self._fox_logo_locator):
                    Wait(self.driver, self.timeout).until(
                        EC.visibility_of_element_located(self._email_input_locator))
                    self._sign_in_window_handle = handle
                    break
            else:
                raise Exception('Popup has not loaded')

    def click_sign_in(self):
        self.driver.find_element(*self._sign_in_locator).click()
        if self.popup:
            Wait(self.driver, self.timeout).until(
                lambda s: self._sign_in_window_handle not in self.driver.window_handles)
            self.switch_to_main_window()

    def sign_in(self, email, password):
        """Signs in using the specified email address and password."""
        self.email = email
        self.login_password = password
        if self.is_element_present(*self._next_button_locator):
            Wait(self.driver, self.timeout).until(
                EC.visibility_of_element_located(self._next_button_locator))
            self.click_next()
        self.click_sign_in()



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import WebDriverWait as Wait

from fxapom.fxapom import WebDriverFxA


class TestLogin(object):

    _fxa_logged_in_indicator_locator = (By.ID, 'loggedin')

    def test_user_can_sign_in(self, selenium, dev_account, click_login, timeout):
        fxa = WebDriverFxA(selenium, timeout)
        fxa.sign_in(dev_account.email, dev_account.password)
        # We sometimes need to wait longer than the standard 10 seconds
        Wait(selenium, timeout).until(
            EC.visibility_of_element_located(self._fxa_logged_in_indicator_locator))



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from marionette_driver import By, expected, Wait

from fxapom.fxapom import WebDriverFxA


class TestLogin(object):

    _fxa_logged_in_indicator_locator = (By.ID, 'loggedin')

    def test_user_can_sign_in(self, base_url, marionette, dev_account, click_login, timeout):
        fxa = WebDriverFxA(marionette, timeout)
        fxa.sign_in(dev_account.email, dev_account.password)
        # We sometimes need to wait longer than the standard 10 seconds
        logged_in = Wait(marionette, timeout).until(
            expected.element_present(*self._fxa_logged_in_indicator_locator))
        Wait(marionette, timeout).until(expected.element_displayed(logged_in))




# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import string
import random
from datetime import datetime

from fxa.core import Client
from fxa.errors import ClientError
from fxa.tests.utils import TestEmailAccount

from selenium.webdriver.remote.webdriver import WebDriver

# Constants for available FxA environments
DEV_URL = 'https://stable.dev.lcip.org/auth/'
PROD_URL = 'https://api.accounts.firefox.com/'

TIMEOUT = 20


class AccountNotFoundException(Exception):
    pass


class WebDriverFxA(object):

    def __init__(self, driver, timeout=TIMEOUT):
        self.driver = driver
        self.timeout = timeout

    def sign_in(self, email=None, password=None):
        """Signs in a user, either with the specified email address and password, or a returning user."""
        if isinstance(self.driver, WebDriver):
            from pages.sign_in import SignIn
        else:
            from pages.marionette.sign_in import MarionetteSignIn as SignIn
        sign_in = SignIn(self.driver, self.timeout)
        sign_in.sign_in(email, password)


class FxATestAccount:
    """A base test class that can be extended by other tests to include utility methods."""

    password = ''.join([random.choice(string.letters) for i in range(8)])

    def __init__(self, url=DEV_URL):
        """ Creates an FxATestAccount object, which includes a verified account.

        :param url: The url for the api host. Defaults to DEV_URL.
        """
        self.url = url
        random_string = ''.join(random.choice(string.ascii_lowercase) for _ in range(12))
        email_pattern = random_string + '@{hostname}'
        self.account = TestEmailAccount(email=email_pattern)
        self.client = Client(self.url)
        # Create and verify the Firefox account
        self.session = self.client.create_account(self.account.email, self.password)
        print 'fxapom created an account for email: %s at %s on %s' % (
            self.account.email, self.url, datetime.now())
        m = self.account.wait_for_email(lambda m: "x-verify-code" in m["headers"])
        if not m:
            raise RuntimeError("Verification email was not received")
        self.session.verify_email_code(m["headers"]["x-verify-code"])

    def __del__(self):
        """ Deletes the Firefox Account that was created during __init__. """
        try:
            self.account.clear()
            self.client.destroy_account(self.email, self.password)
            print 'fxapom deleted the account for email: %s at %s on %s' % (
                self.account.email, self.url, datetime.now())
        except ClientError as err:
            # 'Unknown Account' error is ok - account already deleted
            # https://github.com/mozilla/fxa-auth-server/blob/master/docs/api.md#response-format
            if err.errno == 102:
                return
            raise

    def login(self):
        try:
            session = self.client.login(self.email, self.password)
            return session
        except ClientError as err:
            # 'Unknown Account' error is the only one we care about and will
            # cause us to throw a custom exception
            # https://github.com/mozilla/fxa-auth-server/blob/master/docs/api.md#response-format
            if err.errno == 102:
                raise AccountNotFoundException('FxA Account Not Found')
            raise

    @property
    def email(self):
        return self.account.email

    @property
    def is_verified(self):
        return self.session.get_email_status()['verified']



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import pytest

from marionette_driver import By, Wait
from marionette_driver.marionette import Marionette


@pytest.fixture
def click_login(base_url, marionette, timeout):
    fxa_login_button_locator = (By.CSS_SELECTOR, 'button.signin')
    marionette.navigate('%s/' % base_url)
    Wait(marionette, timeout).until(
        lambda m: m.find_element(*fxa_login_button_locator).is_displayed())
    marionette.find_element(*fxa_login_button_locator).click()


@pytest.fixture
def marionette(request):
    m = Marionette(bin=request.config.option.bin)
    m.start_session()
    m.set_prefs({'signon.rememberSignons': False})
    request.addfinalizer(m.delete_session)
    return m



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this file,
# You can obtain one at http://mozilla.org/MPL/2.0/.

from selenium.common.exceptions import NoSuchElementException

from fxapom.fxapom import TIMEOUT


class Page(object):

    def __init__(self, driver, timeout=TIMEOUT):
        self.driver = driver
        self.timeout = timeout

    def is_element_present(self, *locator):
        try:
            return self.driver.find_element(*locator)
        except NoSuchElementException:
            return False

    def is_element_visible(self, *locator):
        try:
            return self.driver.find_element(*locator).is_displayed()
        except NoSuchElementException:
            return False



from setuptools import setup, find_packages

setup(name='fxapom',
      version='1.7.1',
      description='Mozilla Firefox Accounts Page Object Model',
      long_description=open('README.rst').read(),
      classifiers=[
          'Development Status :: 5 - Production/Stable',
          'Intended Audience :: Developers',
          'License :: OSI Approved :: Mozilla Public License 2.0 (MPL 2.0)',
          'Operating System :: POSIX',
          'Operating System :: Microsoft :: Windows',
          'Operating System :: MacOS :: MacOS X',
          'Topic :: Software Development :: Testing',
          'Topic :: Utilities',
          'Programming Language :: Python',
          'Programming Language :: Python :: 2.6',
          'Programming Language :: Python :: 2.7'],
      keywords='mozilla',
      author='Mozilla Web QA',
      author_email='mozwebqa@mozilla.org',
      url='https://github.com/mozilla/fxapom',
      license='MPL 2.0',
      packages=find_packages(exclude=['ez_setup', 'examples', 'tests']),
      install_requires=['PyFxA==0.0.8'],
      include_package_data=True)
